@@ -24,6 +24,7 @@ case "$1" in
 esac
 
 perl mkfiles.pl
+(cd doc && make -s)
 
 relver=`cat LATEST.VER`
 arcname="putty$arcsuffix"
@@ -33,6 +34,7 @@ find . -name uxarc -prune -o -name . -o \
        -type d -exec mkdir uxarc/$arcname/{} \;
 find . -name uxarc -prune -o \
        -name CVS -prune -o \
+       -name .svn -prune -o \
        -name .cvsignore -prune -o \
        -name '*.zip' -prune -o \
        -name '*.tar.gz' -prune -o \
@@ -42,5 +44,5 @@ if test "x$ver" != "x"; then
    md5sum `find . -name '*.[ch]' -print` > manifest;
    echo "$ver" > version.def)
 fi
-tar -C uxarc -chzf - $arcname > $arcname.tar.gz
+tar -C uxarc -chzof $arcname.tar.gz $arcname
 rm -rf uxarc
