@@ -356,8 +356,8 @@ SockAddr name_lookup(char *host, int port, char **canonicalname,
 
 Socket new_connection(SockAddr addr, char *hostname,
 		      int port, int privport,
-		      int oobinline, int nodelay, Plug plug,
-		      const Config *cfg)
+		      int oobinline, int nodelay, int keepalive,
+		      Plug plug, const Config *cfg)
 {
     static const struct socket_function_table socket_fn_table = {
 	sk_proxy_plug,
@@ -388,7 +388,8 @@ Socket new_connection(SockAddr addr, char *hostname,
 	Socket sret;
 
 	if ((sret = platform_new_connection(addr, hostname, port, privport,
-					    oobinline, nodelay, plug, cfg)) !=
+					    oobinline, nodelay, keepalive,
+					    plug, cfg)) !=
 	    NULL)
 	    return sret;
 
@@ -444,7 +445,7 @@ Socket new_connection(SockAddr addr, char *hostname,
 	 */
 	ret->sub_socket = sk_new(proxy_addr, cfg->proxy_port,
 				 privport, oobinline,
-				 nodelay, (Plug) pplug);
+				 nodelay, keepalive, (Plug) pplug);
 	if (sk_socket_error(ret->sub_socket) != NULL)
 	    return (Socket) ret;
 
@@ -456,7 +457,7 @@ Socket new_connection(SockAddr addr, char *hostname,
     }
 
     /* no proxy, so just return the direct socket */
-    return sk_new(addr, port, privport, oobinline, nodelay, plug);
+    return sk_new(addr, port, privport, oobinline, nodelay, keepalive, plug);
 }
 
 Socket new_listener(char *srcaddr, int port, Plug plug, int local_host_only,
@@ -858,15 +859,16 @@ int proxy_socks5_negotiate (Proxy_Socket p, int change)
 	 *     0x03 = CHAP
 	 */
 
-	char command[4];
+	char command[5];
 	int len;
 
 	command[0] = 5; /* version 5 */
 	if (p->cfg.proxy_username[0] || p->cfg.proxy_password[0]) {
-	    command[1] = 2;	       /* two methods supported: */
 	    command[2] = 0x00;	       /* no authentication */
-	    command[3] = 0x02;	       /* username/password */
-	    len = 4;
+	    len = 3;
+	    proxy_socks5_offerencryptedauth (command, &len);
+	    command[len++] = 0x02;	       /* username/password */
+	    command[1] = len - 2;	/* Number of methods supported */
 	} else {
 	    command[1] = 1;	       /* one methods supported: */
 	    command[2] = 0x00;	       /* no authentication */
@@ -922,7 +924,7 @@ int proxy_socks5_negotiate (Proxy_Socket p, int change)
 	     *    authentication methods:
 	     *     0x00 = no authentication
 	     *     0x01 = GSSAPI
-	     *     0x02 = username/password 
+	     *     0x02 = username/password
 	     *     0x03 = CHAP
 	     *     0xff = no acceptable methods
 	     */
@@ -987,6 +989,12 @@ int proxy_socks5_negotiate (Proxy_Socket p, int change)
 	    p->state = 2;	       /* now proceed as authenticated */
 	}
 
+	if (p->state == 8) {
+	    int ret;
+	    ret = proxy_socks5_handlechap(p);
+	    if (ret) return ret;
+	}
+
 	if (p->state == 2) {
 
 	    /* request format:
@@ -1155,10 +1163,9 @@ int proxy_socks5_negotiate (Proxy_Socket p, int change)
 	}
 
 	if (p->state == 6) {
-	    /* TODO: Handle CHAP authentication */
-	    plug_closing(p->plug, "Proxy error: We don't support CHAP authentication",
-			 PROXY_ERROR_GENERAL, 0);
-	    return 1;
+	    int ret;
+	    ret = proxy_socks5_selectchap(p);
+	    if (ret) return ret;
 	}
 
     }
